@@ -9,7 +9,7 @@ import (
 	"time"
 
 	"github.com/spf13/cobra"
-	"k8s.io/klog"
+	"k8s.io/klog/v2"
 
 	kappsv1 "k8s.io/api/apps/v1"
 	kappsv1beta1 "k8s.io/api/apps/v1beta1"
@@ -26,6 +26,7 @@ import (
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/watch"
+	"k8s.io/apiserver/pkg/storage/names"
 	"k8s.io/cli-runtime/pkg/genericclioptions"
 	"k8s.io/cli-runtime/pkg/printers"
 	"k8s.io/cli-runtime/pkg/resource"
@@ -84,32 +85,34 @@ var (
 		resources that can be used to create pods (such as image stream tags).
 
 		The debug pod is deleted when the remote command completes or the user interrupts
-		the shell.`)
+		the shell.
+	`)
 
 	debugExample = templates.Examples(`
-	  # Debug a currently running deployment by creating a new pod
-		%[1]s deploy/test
+		# Debug a currently running deployment by creating a new pod
+		oc debug deploy/test
 
 		# Debug a node as an administrator
-		%[1]s node/master-1
+		oc debug node/master-1
 
 		# Launch a shell in a pod using the provided image stream tag
-		%[1]s istag/mysql:latest -n openshift
+		oc debug istag/mysql:latest -n openshift
 
-	  # Test running a job as a non-root user
-	  %[1]s job/test --as-user=1000000
+		# Test running a job as a non-root user
+		oc debug job/test --as-user=1000000
 
-	  # Debug a specific failing container by running the env command in the 'second' container
-	  %[1]s daemonset/test -c second -- /bin/env
+		# Debug a specific failing container by running the env command in the 'second' container
+		oc debug daemonset/test -c second -- /bin/env
 
-	  # See the pod that would be created to debug
-	  %[1]s mypod-9xbc -o yaml
+		# See the pod that would be created to debug
+		oc debug mypod-9xbc -o yaml
 
-	  # Debug a resource but launch the debug pod in another namespace.
-	  # Note: Not all resources can be debugged using --to-namespace without modification. For example,
-	  # volumes and serviceaccounts are namespace-dependent. Add '-o yaml' to output the debug pod definition
-	  # to disk.  If necessary, edit the definition then run 'oc debug -f -' or run without --to-namespace.
-	  %[1]s mypod-9xbc --to-namespace testns`)
+		# Debug a resource but launch the debug pod in another namespace.
+		# Note: Not all resources can be debugged using --to-namespace without modification. For example,
+		# volumes and serviceaccounts are namespace-dependent. Add '-o yaml' to output the debug pod definition
+		# to disk.  If necessary, edit the definition then run 'oc debug -f -' or run without --to-namespace.
+		oc debug mypod-9xbc --to-namespace testns
+	`)
 )
 
 type DebugOptions struct {
@@ -153,6 +156,7 @@ type DebugOptions struct {
 	DryRun             bool
 	FullCmdName        string
 	Image              string
+	ImageStream        string
 	ToNamespace        string
 
 	// IsNode is set after we see the object we're debugging.  We use it to be able to print pertinent advice.
@@ -178,13 +182,13 @@ func NewDebugOptions(streams genericclioptions.IOStreams) *DebugOptions {
 }
 
 // NewCmdDebug creates a command for debugging pods.
-func NewCmdDebug(fullName string, f kcmdutil.Factory, streams genericclioptions.IOStreams) *cobra.Command {
+func NewCmdDebug(f kcmdutil.Factory, streams genericclioptions.IOStreams) *cobra.Command {
 	o := NewDebugOptions(streams)
 	cmd := &cobra.Command{
 		Use:     "debug RESOURCE/NAME [ENV1=VAL1 ...] [-c CONTAINER] [flags] [-- COMMAND]",
 		Short:   "Launch a new instance of a pod for debugging",
 		Long:    debugLong,
-		Example: fmt.Sprintf(debugExample, fmt.Sprintf("%s debug", fullName)),
+		Example: debugExample,
 		Run: func(cmd *cobra.Command, args []string) {
 			kcmdutil.CheckErr(o.Complete(cmd, f, args))
 			kcmdutil.CheckErr(o.Validate())
@@ -218,6 +222,7 @@ func NewCmdDebug(fullName string, f kcmdutil.Factory, streams genericclioptions.
 	cmd.Flags().BoolVar(&o.AsRoot, "as-root", o.AsRoot, "If true, try to run the container as the root user")
 	cmd.Flags().Int64Var(&o.AsUser, "as-user", o.AsUser, "Try to run the container as a specific user UID (note: admins may limit your ability to use this flag)")
 	cmd.Flags().StringVar(&o.Image, "image", o.Image, "Override the image used by the targeted container.")
+	cmd.Flags().StringVar(&o.ImageStream, "image-stream", o.ImageStream, "Specify an image stream (namespace/name:tag) containing a debug image to run.")
 	cmd.Flags().StringVar(&o.ToNamespace, "to-namespace", o.ToNamespace, "Override the namespace to create the pod into (instead of using --namespace).")
 	cmd.Flags().BoolVar(&o.PreservePod, "preserve-pod", o.PreservePod, "If true, the pod will not be deleted after the debug command exits.")
 
@@ -373,11 +378,49 @@ func (o *DebugOptions) RunDebug() error {
 	if err != nil {
 		klog.V(4).Infof("Unable to get exact template, but continuing with fallback: %v", err)
 	}
+	var debugNodeNS *corev1.Namespace
+	generateName := names.SimpleNameGenerator.GenerateName("openshift-debug-node-")
+	// create namespace only if debugging a node
+	if o.IsNode && len(o.ToNamespace) == 0 && !o.ExplicitNamespace {
+		if !o.Attach.Quiet {
+			fmt.Fprintf(o.ErrOut, "Creating debug namespace/%s ...\n", generateName)
+		}
+		debugNodeNS, err = o.CoreClient.Namespaces().Create(context.TODO(), &corev1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				GenerateName: generateName,
+				Labels: map[string]string{
+					"openshift.io/run-level": "0",
+				},
+				Annotations: map[string]string{
+					"openshift.io/node-selector": "",
+				},
+			},
+		}, metav1.CreateOptions{})
+		if err != nil {
+			return err
+		}
+	}
+	defer func() {
+		if debugNodeNS != nil {
+			if !o.Attach.Quiet {
+				fmt.Fprintf(o.ErrOut, "Removing debug namespace/%s ...\n", generateName)
+			}
+
+			if err := o.CoreClient.Namespaces().Delete(context.TODO(), debugNodeNS.Name, metav1.DeleteOptions{}); err != nil {
+				fmt.Fprintf(o.ErrOut, "%v\n", err)
+				return
+			}
+		}
+	}()
+
 	pod := &corev1.Pod{
 		ObjectMeta: template.ObjectMeta,
 		Spec:       template.Spec,
 	}
 	ns := infos[0].Namespace
+	if debugNodeNS != nil {
+		ns = debugNodeNS.Name
+	}
 	if len(ns) == 0 {
 		ns = o.Namespace
 	}
@@ -869,13 +912,19 @@ func (o *DebugOptions) approximatePodTemplateForObject(object runtime.Object) (*
 		}
 		image := o.Image
 		if len(o.Image) == 0 {
-			istag, err := o.ImageClient.ImageStreamTags("openshift").Get(context.TODO(), "tools:latest", metav1.GetOptions{})
-			if err == nil && len(istag.Image.DockerImageReference) > 0 {
-				image = istag.Image.DockerImageReference
+			imageStream := o.ImageStream
+			if len(o.ImageStream) == 0 {
+				imageStream = "openshift/tools:latest"
+			}
+			if imageFromStream, err := o.resolveImageStreamTagString(imageStream); err == nil {
+				image = imageFromStream
+			} else {
+				klog.V(2).Infof("Unable to resolve image stream '%v': %v", imageStream, err)
 			}
 		}
 		if len(image) == 0 {
-			image = "registry.redhat.io/rhel7/support-tools"
+			klog.V(2).Infof("Falling to 'registry.redhat.io/rhel8/support-tools' image")
+			image = "registry.redhat.io/rhel8/support-tools"
 		}
 		zero := int64(0)
 		isTrue := true
@@ -1046,3 +1095,38 @@ func setNodeName(template *corev1.PodTemplateSpec, nodeName string, overrideWhen
 	}
 	return template
 }
+
+func (o *DebugOptions) resolveImageStreamTagString(s string) (string, error) {
+	namespace, name, tag := parseImageStreamTagString(s)
+	if len(namespace) == 0 {
+		return "", fmt.Errorf("expected namespace/name:tag")
+	}
+	return o.resolveImageStreamTag(namespace, name, tag)
+}
+
+func parseImageStreamTagString(s string) (string, string, string) {
+	var namespace, nameAndTag string
+	parts := strings.SplitN(s, "/", 2)
+	switch len(parts) {
+	case 2:
+		namespace = parts[0]
+		nameAndTag = parts[1]
+	case 1:
+		nameAndTag = parts[0]
+	}
+	name, tag, _ := imageutil.SplitImageStreamTag(nameAndTag)
+	return namespace, name, tag
+}
+
+func (o *DebugOptions) resolveImageStreamTag(namespace, name, tag string) (string, error) {
+	imageStream, err := o.ImageClient.ImageStreams(namespace).Get(context.TODO(), name, metav1.GetOptions{})
+	if err != nil {
+		return "", err
+	}
+	var image string
+	var ok bool
+	if image, ok = imageutil.ResolveLatestTaggedImage(imageStream, tag); !ok {
+		return "", fmt.Errorf("unable to resolve the imagestream tag %s/%s:%s", namespace, name, tag)
+	}
+	return image, nil
+}
